package uk.co.gataki.mapdroid;

import android.app.Activity;
import android.app.AlertDialog;
import android.os.Bundle;
import android.widget.Button;
import android.widget.Toast;
import android.view.View;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.location.Location;
import android.location.LocationManager;
import android.location.LocationListener;
import android.content.Context;
import android.content.DialogInterface;
import android.util.Log;
import java.util.ArrayList;
import java.util.List;
import java.util.HashMap;
import android.content.Intent;
import android.preference.PreferenceManager;

public class MapDroidActivity extends Activity
{
	private MapView mapView;

	private static final String TAG = "MDA";

    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);

		mapView = (MapView) findViewById(R.id.mapView);
		mapView.setBackgroundColor(0xff000000);
		mapView.SetParent(this);
		mapView.SetFollowPosition(true);
		mapView.SetPreferences(PreferenceManager.getDefaultSharedPreferences(getBaseContext()));

        Button btnZoomIn = (Button) findViewById(R.id.btnZoomIn);
        Button btnZoomOut = (Button) findViewById(R.id.btnZoomOut);
        Button btnFollow = (Button) findViewById(R.id.btnFollow);

        btnZoomIn.setOnClickListener(ZoomInListener);
        btnZoomOut.setOnClickListener(ZoomOutListener);
        btnFollow.setOnClickListener(FollowListener);

        LocationManager locationManager = (LocationManager)
        	this.getSystemService(Context.LOCATION_SERVICE);
        mapView.SetLocation(locationManager.getLastKnownLocation(LocationManager.NETWORK_PROVIDER));
        locationManager.requestLocationUpdates(LocationManager.NETWORK_PROVIDER,
        		0, 0, LocationChangeListener);
        locationManager.requestLocationUpdates(LocationManager.GPS_PROVIDER,
        		0, 0, LocationChangeListener);
    }

	public void onFocusChange (View v, boolean hasFocus)
	{
		if ( ! hasFocus)
		{
			mapView.StorePreferences();
		}
	}

	public void UpdateFollowDisplay(boolean bIsFollowing)
	{
		Button btnFollow = (Button) findViewById(R.id.btnFollow);
		if (bIsFollowing)
			btnFollow.setVisibility(View.GONE);
		else
			btnFollow.setVisibility(View.VISIBLE);
	}

	private View.OnClickListener ZoomInListener = new View.OnClickListener()
	{
		public void onClick(View v)
		{
			mapView.ZoomIn();
		}
	};
	private View.OnClickListener ZoomOutListener = new View.OnClickListener()
	{
		public void onClick(View v)
		{
			mapView.ZoomOut();
		}
	};

	private View.OnClickListener FollowListener = new View.OnClickListener()
	{
		public void onClick(View v)
		{
			mapView.SetFollowPosition(true);
		}
	};

	private LocationListener LocationChangeListener = new LocationListener()
	{
		public void onLocationChanged(Location location)
		{
			mapView.SetLocation(location);
		}

		public void onStatusChanged(String provider, int status, Bundle extras)
		{
		}

		public void onProviderEnabled(String provider)
		{
		}

		public void onProviderDisabled(String provider)
		{
		}
	};

	private void SelectSources()
	{
		HashMap<Integer,String> hshSources = mapView.GetSourceList();
		HashMap<Integer,Boolean> hshSourceStates = mapView.GetSourceStates();

		if (hshSources.size() == 0)
		{
			return;
		}
		
		CharSequence[] lstSourceNames = new String[hshSources.size()];
		boolean[] lstSourceStates = new boolean[hshSources.size()];
		int[] lstIndices = new int[hshSources.size()];

		int index = 0;
		for (int i: hshSources.keySet())
		{
			lstSourceNames[index] = (String) hshSources.get(new Integer(i));
			lstSourceStates[index] = (boolean) hshSourceStates.get(new Integer(i));
			lstIndices[index] = i;
			index += 1;
		}

		// Nasty casting to keep the java compiler happy
		final CharSequence[] items = lstSourceNames;
		final boolean[] states = lstSourceStates;
		final int[] indices = lstIndices;

		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle("Choose Sources");
		builder.setMultiChoiceItems(items, states, new DialogInterface.OnMultiChoiceClickListener()
				{
					public void onClick(DialogInterface dialog, int item, boolean isChecked)
					{
						Toast.makeText(getApplicationContext(), items[item], Toast.LENGTH_SHORT).show();
						if (isChecked)
						{
							mapView.EnableSource(indices[item]);
						}
						else
						{
							mapView.DisableSource(indices[item]);
						}
					}
				});
		builder.setPositiveButton("OK", new DialogInterface.OnClickListener()
				{
					public void onClick(DialogInterface dialog, int which)
					{
						return;
					}
				}); 
		AlertDialog alert = builder.create();
		alert.show();
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu)
	{
		MenuInflater inflater = getMenuInflater();
		inflater.inflate(R.menu.options_menu, menu);
		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) 
	{
		switch (item.getItemId()) 
		{
			case R.id.settings:
				Log.d(TAG, "Showing Settings");
				startActivity(new Intent(this, SettingsDialog.class));
				return true;

			case R.id.sources:
				Log.d(TAG, "Showing source selection dialog");
				SelectSources();
				return true;

			case R.id.quit:
				Log.d(TAG, "Finishing");
				finish();
				return true;

			default:
				return super.onOptionsItemSelected(item);
		}
	}

}
